import os.path
from typing import Optional

from django import template
from django.conf import settings
from django.forms.utils import flatatt
from django.templatetags.static import static
from django.utils.safestring import mark_safe

from sri.algorithm import DEFAULT_ALGORITHM, Algorithm
from sri.integrity import calculate_integrity_of_static

USE_SRI = getattr(settings, "USE_SRI", not settings.DEBUG)

register = template.Library()


def sri_js(attrs: dict, path: str, algorithm: Algorithm):
    attrs.update({"type": "text/javascript", "src": static(path)})
    return mark_safe(f"<script{flatatt(attrs)}></script>")


def sri_css(attrs: dict, path: str, algorithm: Algorithm):
    attrs.update({"rel": "stylesheet", "type": "text/css", "href": static(path)})
    return mark_safe(f"<link{flatatt(attrs)}/>")


EXTENSIONS = {"js": sri_js, "css": sri_css}


@register.simple_tag
def sri_static(path: str, algorithm: Optional[str] = None):
    algorithm_type = Algorithm(algorithm or DEFAULT_ALGORITHM)
    extension = os.path.splitext(path)[1][1:]
    sri_method = EXTENSIONS[extension]
    attrs = {}
    if USE_SRI:
        attrs.update(
            {
                "integrity": calculate_integrity_of_static(path, algorithm_type),
                "crossorigin": "anonymous",
            }
        )
    return sri_method(attrs, path, algorithm_type)


@register.simple_tag
def sri_integrity_static(path: str, algorithm: Optional[str] = None):
    return calculate_integrity_of_static(
        path, Algorithm(algorithm or DEFAULT_ALGORITHM)
    )



from pathlib import Path

from django.contrib.staticfiles.finders import find as find_static_file
from django.core.cache import DEFAULT_CACHE_ALIAS, caches
from django.core.cache.backends.base import InvalidCacheBackendError


def get_static_path(path: str) -> Path:
    """
    Resolves a path commonly passed to `{% static %}` into a filesystem path
    """
    static_file_path = find_static_file(path)
    if static_file_path is None:
        raise FileNotFoundError(path)
    return Path(static_file_path)


def get_cache():
    try:
        return caches["sri"]
    except InvalidCacheBackendError:
        return caches[DEFAULT_CACHE_ALIAS]



import base64
import hashlib
from functools import lru_cache
from pathlib import Path

from sri.algorithm import Algorithm
from sri.utils import get_cache

HASHERS = {
    Algorithm.SHA256: hashlib.sha256,
    Algorithm.SHA384: hashlib.sha384,
    Algorithm.SHA512: hashlib.sha512,
}


def calculate_hash(path: Path, algorithm: Algorithm) -> str:
    cache = get_cache()
    cache_key = get_cache_key(path, algorithm)
    file_hash = cache.get(cache_key)
    if file_hash is None:
        # Cache miss, do the calculation
        hasher = HASHERS[algorithm]()
        with path.open("rb") as f:
            while True:
                data = f.read(hasher.block_size)
                if not data:
                    break
                hasher.update(data)
        file_hash = base64.b64encode(hasher.digest()).decode()
        cache.set(cache_key, file_hash)
    return file_hash


@lru_cache()
def get_cache_key(path: Path, algorithm: Algorithm) -> str:
    path_hash = hashlib.sha1(str(path).encode()).hexdigest()
    return f"sri-{path_hash}-{algorithm.value}"



from pathlib import Path

from sri.algorithm import DEFAULT_ALGORITHM, Algorithm
from sri.hashers import calculate_hash
from sri.utils import get_static_path


def calculate_integrity(path: Path, algorithm: Algorithm = DEFAULT_ALGORITHM) -> str:
    return f"{algorithm.value}-{calculate_hash(path, algorithm)}"


def calculate_integrity_of_static(
    static_path: str, algorithm: Algorithm = DEFAULT_ALGORITHM
) -> str:
    return calculate_integrity(get_static_path(static_path), algorithm)
